#include "BVQXPluginInterface.h"

#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif
#include <stdio.h>
#include <stdlib.h>

#ifndef OS_WIN32
#include <dlfcn.h>
#endif

#ifdef OS_WIN32
#define GETSYMADDRESS GetProcAddress
#else
#define GETSYMADDRESS dlsym
#endif

// general functions
GetVersionOfBrainVoyagerQXPrototype qxGetVersionOfBrainVoyagerQX;
GetBrainVoyagerQXPathPrototype qxGetBrainVoyagerQXPath;
GetPluginsPathPrototype qxGetPluginsPath;

// input / output
ShowLogTabPrototype qxShowLogTab;
LogTextPrototype qxLogText;
MessageBoxPrototype qxMessageBox;
YesNoMessageBoxPrototype qxYesNoMessageBox;
GetIntegerInputPrototype qxGetIntegerInput;
GetFloatInputPrototype qxGetFloatInput;
GetTextInputPrototype qxGetTextInput;
GetOpenFileNamePrototype qxGetOpenFileName;
GetSaveFileNamePrototype qxGetSaveFileName;

// GUI
UpdateActiveWindowPrototype qxUpdateActiveWindow;
ShowBusyCursorPrototype qxShowBusyCursor;
StopBusyCursorPrototype qxStopBusyCursor;

// Docs
GetNumberOfDocsInWorkspacePrototype qxGetNumberOfDocsInWorkspace;  // v1.10.4
GetDocumentTypePrototype qxGetDocumentType;  // v1.10.4
GetDocumentTypeNamePrototype qxGetDocumentTypeName;  // v1.10.4
GetIDOfCurrentDocumentPrototype qxGetIDOfCurrentDocument;  // v1.10.4
SetAsCurrentDocumentPrototype qxSetAsCurrentDocument;  // v1.10.4
CalculateIAMatrixPrototype qxCalculateIAMatrix;

// FMR
GetHeaderOfCurrentFMRPrototype qxGetHeaderOfCurrentFMR;
SetHeaderOfCurrentFMRPrototype qxSetHeaderOfCurrentFMR;
GetSTCDataOfCurrentFMRPrototype qxGetSTCDataOfCurrentFMR;
SaveFMRPrototype qxSaveFMR;
SaveFMRAndSTCPrototype qxSaveFMRAndSTC;
GetScannerPositionHeaderOfCurrentFMRPrototype qxGetScannerPositionHeaderOfCurrentFMR;  // v1.10.4
SetScannerPositionHeaderOfCurrentFMRPrototype qxSetScannerPositionHeaderOfCurrentFMR;  // v1.10.4

// DMR
GetHeaderOfCurrentDMRPrototype qxGetHeaderOfCurrentDMR;
SetHeaderOfCurrentDMRPrototype qxSetHeaderOfCurrentDMR;
GetDWIDataOfCurrentDMRPrototype qxGetDWIDataOfCurrentDMR;
ConvertDWIFormatOfCurrentDMRPrototype qxConvertDWIFormatOfCurrentDMR;
SaveDMRPrototype qxSaveDMR;
SaveDMRAndDWIPrototype qxSaveDMRAndDWI;

// VMR
GetCurrentVMRPrototype qxGetCurrentVMR;
GetMainHeaderOfCurrentVMRPrototype qxGetMainHeaderOfCurrentVMR;
SetMainHeaderOfCurrentVMRPrototype qxSetMainHeaderOfCurrentVMR;
GetScannerPositionHeaderOfCurrentVMRPrototype qxGetScannerPositionHeaderOfCurrentVMR;
SetScannerPositionHeaderOfCurrentVMRPrototype qxSetScannerPositionHeaderOfCurrentVMR;
GetPastTransformationsHeaderOfCurrentVMRPrototype qxGetPastTransformationsHeaderOfCurrentVMR;
SetPastTransformationsHeaderOfCurrentVMRPrototype qxSetPastTransformationsHeaderOfCurrentVMR;
ReframeCurrentVMRPrototype qxReframeCurrentVMR;
GetCrossPositionOfCurrentVMRPrototype qxGetCrossPositionOfCurrentVMR;

// nrVMP
GetNRVMPsOfCurrentVMRPrototype qxGetNRVMPsOfCurrentVMR;
GetNRVMPOfCurrentVMRPrototype qxGetNRVMPOfCurrentVMR;
SetNRVMPParametersOfCurrentVMRPrototype qxSetNRVMPParametersOfCurrentVMR;
LoadNRVMPsToCurrentVMRPrototype qxLoadNRVMPsToCurrentVMR;
SaveNRVMPsOfCurrentVMRPrototype qxSaveNRVMPsOfCurrentVMR;
CreateNRVMPsForCurrentVMRPrototype qxCreateNRVMPsForCurrentVMR;
DeleteNRVMPsOfCurrentVMRPrototype qxDeleteNRVMPsOfCurrentVMR;
SetNRVMPsParamsDisplayRangePrototype qxSetNRVMPsParamsDisplayRange;

// arVMP
GetARVMPsOfCurrentVMRPrototype qxGetARVMPsOfCurrentVMR;
GetARVMPOfCurrentVMRPrototype qxGetARVMPOfCurrentVMR;
SetARVMPParametersOfCurrentVMRPrototype qxSetARVMPParametersOfCurrentVMR;
LoadARVMPsToCurrentVMRPrototype qxLoadARVMPsToCurrentVMR;
SaveARVMPsOfCurrentVMRPrototype qxSaveARVMPsOfCurrentVMR;
CreateARVMPsForCurrentVMRPrototype qxCreateARVMPsForCurrentVMR;
DeleteARVMPsOfCurrentVMRPrototype qxDeleteARVMPsOfCurrentVMR;

// VTC
LinkVTCToCurrentVMRPrototype qxLinkVTCToCurrentVMR;
GetVTCOfCurrentVMRPrototype qxGetVTCOfCurrentVMR;
SetVTCParametersOfCurrentVMRPrototype qxSetVTCParametersOfCurrentVMR;
CreateVTCForCurrentVMRPrototype qxCreateVTCForCurrentVMR;
SaveVTCOfCurrentVMRPrototype qxSaveVTCOfCurrentVMR;

// Multi-VTC
ClearMultiVTCAnalysisPrototype qxClearMultiVTCAnalysis;
AddVTCRefForMultiVTCAnalysisPrototype qxAddVTCRefForMultiVTCAnalysis;
GetVoxelTimeCourseOfVTCPrototype qxGetVoxelTimeCourseOfVTC;
GetVoxelTimeCourseOfVTCsPrototype qxGetVoxelTimeCourseOfVTCs;
GetVoxelCoordsTimeCourseOfVTCPrototype qxGetVoxelCoordsTimeCourseOfVTC;
GetVoxelCoordsTimeCourseOfVTCsPrototype qxGetVoxelCoordsTimeCourseOfVTCs;

// GLM
GetGLMOfCurrentVMRPrototype qxGetGLMOfCurrentVMR;

// SRF
GetScenePrototype qxGetScene;
GetCurrentSRFPrototype qxGetCurrentSRF;
ReleaseSRFAccessPrototype qxReleaseSRFAccess;
SaveCurrentSRFPrototype qxSaveCurrentSRF;
SetCurrentSRFPrototype qxSetCurrentSRF;

// MTC
LinkMTCToCurrentSRFPrototype qxLinkMTCToCurrentSRF;
GetMTCOfCurrentSRFPrototype qxGetMTCOfCurrentSRF;
SetMTCParametersOfCurrentSRFPrototype qxSetMTCParametersOfCurrentSRF;
CreateMTCForCurrentSRFPrototype qxCreateMTCForCurrentSRF;
SaveMTCOfCurrentSRFPrototype qxSaveMTCOfCurrentSRF;

// Multi-MTC (file-based read access)
ClearMultiMTCRefsOfCurrentSRFPrototype qxClearMultiMTCRefsOfCurrentSRF;
AddMultiMTCRefToCurrentSRFPrototype qxAddMultiMTCRefToCurrentSRF;
GetVertexTimeCourseOfMTCPrototype qxGetVertexTimeCourseOfMTC;
GetVertexTimeCourseOfAllMTCsPrototype qxGetVertexTimeCourseOfAllMTCs;

// SMP
GetSMPsOfCurrentSRFPrototype qxGetSMPsOfCurrentSRF;
GetSMPOfCurrentSRFPrototype qxGetSMPOfCurrentSRF;
SetSMPParametersOfCurrentSRFPrototype qxSetSMPParametersOfCurrentSRF;
LoadSMPsToCurrentSRFPrototype qxLoadSMPsToCurrentSRF;
SaveSMPsOfCurrentSRFPrototype qxSaveSMPsOfCurrentSRF;
CreateSMPForCurrentSRFPrototype qxCreateSMPForCurrentSRF;
DeleteSMPsOfCurrentSRFPrototype qxDeleteSMPsOfCurrentSRF;
OverlaySMPsOnSRFPrototype qxOverlaySMPsOnSRF;

// SSM
GetSSMOfCurrentSRFPrototype qxGetSSMOfCurrentSRF;
LoadSSMToCurrentSRFPrototype qxLoadSSMToCurrentSRF;
SaveSSMOfCurrentSRFPrototype qxSaveSSMOfCurrentSRF;
CreateSSMForCurrentSRFPrototype qxCreateSSMForCurrentSRF;
DeleteSSMOfCurrentSRFPrototype qxDeleteSSMOfCurrentSRF;

// MSK
LoadMaskToCurrentVMRPrototype qxLoadMaskToCurrentVMR;
GetMaskOfCurrentVMRPrototype qxGetMaskOfCurrentVMR;

// PRT
ClearProtocolPrototype qxClearProtocol;
LoadProtocolPrototype qxLoadProtocol;
SaveProtocolPrototype qxSaveProtocol;
GetProtocolPrototype qxGetProtocol;
SetProtocolPrototype qxSetProtocol;
GetProtocolConditionPrototype qxGetProtocolCondition;
AddProtocolConditionPrototype qxAddProtocolCondition;
AddProtocolConditionIntervalPrototype qxAddProtocolConditionInterval;

// PRED, DM, HRF
ClearDesignMatrixPrototype qxClearDesignMatrix;
AddPredictorPrototype qxAddPredictor;
SetPredictorValuesFromConditionPrototype qxSetPredictorValuesFromCondition;
SetPredictorValuesFromConditionNamePrototype qxSetPredictorValuesFromConditionName;
ApplyTwoGammaHRFToPredictorPrototype qxApplyTwoGammaHRFToPredictor;
ApplyTwoGammaHRFToPredictorWithNamePrototype qxApplyTwoGammaHRFToPredictorWithName;
ApplyBoyntonHRFToPredictorPrototype qxApplyBoyntonHRFToPredictor;
ApplyBoyntonHRFToPredictorWithNamePrototype qxApplyBoyntonHRFToPredictorWithName;
GetPredictorPrototype qxGetPredictor;
GetPredictorByNamePrototype qxGetPredictorByName;
LoadSingleStudyGLMDesignMatrixPrototype qxLoadSingleStudyGLMDesignMatrix;
SaveSingleStudyGLMDesignMatrixPrototype qxSaveSingleStudyGLMDesignMatrix;

// VOI
GetVOIsPrototype qxGetVOIs;
GetVOIPrototype qxGetVOI;

// POI
GetPOIsPrototype qxGetPOIs;
GetPOIPrototype qxGetPOI;

// dialog access
GetDlgStringParameterPrototype qxGetDlgStringParameter;
GetDlgIntegerParameterPrototype qxGetDlgIntegerParameter;
GetDlgFloatParameterPrototype qxGetDlgFloatParameter;
SetDlgStringOutputParameterPrototype qxSetDlgStringOutputParameter;
SetDlgIntegerOutputParameterPrototype qxSetDlgIntegerOutputParameter;
SetDlgFloatOutputParameterPrototype qxSetDlgFloatOutputParameter;
LaunchICADialogPrototype qxLaunchICADialog;


bool InitQXAccess()
{
#ifdef OS_WIN32

	HMODULE hndl = GetModuleHandleA("BrainVoyagerQX.exe");

	if(hndl == 0)
		return false;

#else

	// handle Mac and Linux
	
	void *hndl = dlopen(NULL, RTLD_LAZY);  // NULL pointer to get handle to main process
	if(hndl == NULL)
		return false;
	
#endif

	//
	// general functions
	//
	qxGetVersionOfBrainVoyagerQX = (GetVersionOfBrainVoyagerQXPrototype)GETSYMADDRESS( hndl, "qxGetVersionOfBrainVoyagerQX" );
	if(qxGetVersionOfBrainVoyagerQX == NULL) return false;

	// IMPORTANT: If we do not get a pointer, we do not return false
	//  To use specific commands, a) you must check the obtained version string and use only those commands available in that version
	//                         or b) Check whether the function pointers are non-NULL for each command you want to use in your code

	qxGetBrainVoyagerQXPath = (GetBrainVoyagerQXPathPrototype)GETSYMADDRESS( hndl, "qxGetBrainVoyagerQXPath" );  // if(qxGetBrainVoyagerQXPath == NULL) return false;
	qxGetPluginsPath = (GetPluginsPathPrototype)GETSYMADDRESS( hndl, "qxGetPluginsPath" );

	//
	// input and display functions
	//
	qxShowLogTab = (ShowLogTabPrototype)GETSYMADDRESS( hndl, "qxShowLogTab" );  // if(qxShowLogTab == NULL) return false;
	qxLogText = (LogTextPrototype)GETSYMADDRESS( hndl, "qxLogText" );
	qxMessageBox = (MessageBoxPrototype)GETSYMADDRESS( hndl, "qxMessageBox" );
	qxYesNoMessageBox = (YesNoMessageBoxPrototype)GETSYMADDRESS( hndl, "qxYesNoMessageBox" );
	qxGetIntegerInput = (GetIntegerInputPrototype)GETSYMADDRESS( hndl, "qxGetIntegerInput" );
	qxGetFloatInput = (GetFloatInputPrototype)GETSYMADDRESS( hndl, "qxGetFloatInput" );
	qxGetTextInput = (GetTextInputPrototype)GETSYMADDRESS( hndl, "qxGetTextInput" );
	qxGetOpenFileName = (GetOpenFileNamePrototype)GETSYMADDRESS( hndl, "qxGetOpenFileName" );
	qxGetSaveFileName = (GetSaveFileNamePrototype)GETSYMADDRESS( hndl, "qxGetSaveFileName" );
	qxUpdateActiveWindow = (UpdateActiveWindowPrototype)GETSYMADDRESS( hndl, "qxUpdateActiveWindow" );
	qxShowBusyCursor = (ShowBusyCursorPrototype)GETSYMADDRESS( hndl, "qxShowBusyCursor" );
	qxStopBusyCursor = (StopBusyCursorPrototype)GETSYMADDRESS( hndl, "qxStopBusyCursor" );

	//
	// access to core data
	//

	// Docs
	qxGetNumberOfDocsInWorkspace = (GetNumberOfDocsInWorkspacePrototype)GETSYMADDRESS( hndl, "qxGetNumberOfDocsInWorkspace" );
	qxGetDocumentType = (GetDocumentTypePrototype)GETSYMADDRESS( hndl, "qxGetDocumentType" );
	qxGetDocumentTypeName = (GetDocumentTypeNamePrototype)GETSYMADDRESS( hndl, "qxGetDocumentTypeName" );
	qxGetIDOfCurrentDocument = (GetIDOfCurrentDocumentPrototype)GETSYMADDRESS( hndl, "qxGetIDOfCurrentDocument" );
	qxSetAsCurrentDocument = (SetAsCurrentDocumentPrototype)GETSYMADDRESS( hndl, "qxSetAsCurrentDocument" );
	qxCalculateIAMatrix = (CalculateIAMatrixPrototype)GETSYMADDRESS( hndl, "qxCalculateIAMatrix" );

	// FMR
	qxGetHeaderOfCurrentFMR = (GetHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetHeaderOfCurrentFMR" );
	qxSetHeaderOfCurrentFMR = (SetHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxSetHeaderOfCurrentFMR" );
	qxGetSTCDataOfCurrentFMR = (GetSTCDataOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetSTCDataOfCurrentFMR" );
	qxSaveFMR = (SaveFMRPrototype)GETSYMADDRESS( hndl, "qxSaveFMR" );
	qxSaveFMRAndSTC = (SaveFMRAndSTCPrototype)GETSYMADDRESS( hndl, "qxSaveFMRAndSTC" );
	qxGetScannerPositionHeaderOfCurrentFMR = (GetScannerPositionHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxGetScannerPositionHeaderOfCurrentFMR" );
	qxSetScannerPositionHeaderOfCurrentFMR = (SetScannerPositionHeaderOfCurrentFMRPrototype)GETSYMADDRESS( hndl, "qxSetScannerPositionHeaderOfCurrentFMR" );

	// DMR
	qxGetHeaderOfCurrentDMR = (GetHeaderOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxGetHeaderOfCurrentDMR" );
	qxSetHeaderOfCurrentDMR = (SetHeaderOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxSetHeaderOfCurrentDMR" );
	qxGetDWIDataOfCurrentDMR = (GetDWIDataOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxGetDWIDataOfCurrentDMR" );
	qxConvertDWIFormatOfCurrentDMR = (ConvertDWIFormatOfCurrentDMRPrototype)GETSYMADDRESS( hndl, "qxConvertDWIFormatOfCurrentDMR" );
	qxSaveDMR = (SaveDMRPrototype)GETSYMADDRESS( hndl, "qxSaveDMR" );
	qxSaveDMRAndDWI = (SaveDMRAndDWIPrototype)GETSYMADDRESS( hndl, "qxSaveDMRAndDWI" );

	// VMR
	qxGetCurrentVMR = (GetCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetCurrentVMR" );
	qxGetMainHeaderOfCurrentVMR = (GetMainHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetMainHeaderOfCurrentVMR" );
	qxSetMainHeaderOfCurrentVMR = (SetMainHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetMainHeaderOfCurrentVMR" );
	qxGetScannerPositionHeaderOfCurrentVMR = (GetScannerPositionHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetScannerPositionHeaderOfCurrentVMR" );
	qxSetScannerPositionHeaderOfCurrentVMR = (SetScannerPositionHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetScannerPositionHeaderOfCurrentVMR" );
	qxGetPastTransformationsHeaderOfCurrentVMR = (GetPastTransformationsHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetPastTransformationsHeaderOfCurrentVMR" );
	qxSetPastTransformationsHeaderOfCurrentVMR = (SetPastTransformationsHeaderOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetPastTransformationsHeaderOfCurrentVMR" );
	qxReframeCurrentVMR = (ReframeCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxReframeCurrentVMR" );
	qxGetCrossPositionOfCurrentVMR = (GetCrossPositionOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetCrossPositionOfCurrentVMR" );

	// nrVMP
	qxGetNRVMPsOfCurrentVMR = (GetNRVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetNRVMPsOfCurrentVMR" );
	qxGetNRVMPOfCurrentVMR = (GetNRVMPOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetNRVMPOfCurrentVMR" );
	qxSetNRVMPParametersOfCurrentVMR = (SetNRVMPParametersOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetNRVMPParametersOfCurrentVMR" );
	qxLoadNRVMPsToCurrentVMR = (LoadNRVMPsToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadNRVMPsToCurrentVMR" );
	qxSaveNRVMPsOfCurrentVMR = (SaveNRVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveNRVMPsOfCurrentVMR" );
	qxCreateNRVMPsForCurrentVMR = (CreateNRVMPsForCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxCreateNRVMPsForCurrentVMR" );
	qxDeleteNRVMPsOfCurrentVMR = (DeleteNRVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxDeleteNRVMPsOfCurrentVMR" );
	qxSetNRVMPsParamsDisplayRange = (SetNRVMPsParamsDisplayRangePrototype)GETSYMADDRESS( hndl, "qxSetNRVMPsParamsDisplayRange" );

	// arVMP
	qxGetARVMPsOfCurrentVMR = (GetARVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetARVMPsOfCurrentVMR" );
	qxGetARVMPOfCurrentVMR = (GetARVMPOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetARVMPOfCurrentVMR" );
	qxSetARVMPParametersOfCurrentVMR = (SetARVMPParametersOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetARVMPParametersOfCurrentVMR" );
	qxLoadARVMPsToCurrentVMR = (LoadARVMPsToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadARVMPsToCurrentVMR" );
	qxSaveARVMPsOfCurrentVMR = (SaveARVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveARVMPsOfCurrentVMR" );
	qxCreateARVMPsForCurrentVMR = (CreateARVMPsForCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxCreateARVMPsForCurrentVMR" );
	qxDeleteARVMPsOfCurrentVMR = (DeleteARVMPsOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxDeleteARVMPsOfCurrentVMR" );

	// VTC
	qxLinkVTCToCurrentVMR = (LinkVTCToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLinkVTCToCurrentVMR" );
	qxGetVTCOfCurrentVMR = (GetVTCOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetVTCOfCurrentVMR" );
	qxSetVTCParametersOfCurrentVMR = (SetVTCParametersOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSetVTCParametersOfCurrentVMR" );
	qxCreateVTCForCurrentVMR = (CreateVTCForCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxCreateVTCForCurrentVMR" );
	qxSaveVTCOfCurrentVMR = (SaveVTCOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxSaveVTCOfCurrentVMR" );

	// Multi-VTC
	qxClearMultiVTCAnalysis = (ClearMultiVTCAnalysisPrototype)GETSYMADDRESS( hndl, "qxClearMultiVTCAnalysis" );
	qxAddVTCRefForMultiVTCAnalysis = (AddVTCRefForMultiVTCAnalysisPrototype)GETSYMADDRESS( hndl, "qxAddVTCRefForMultiVTCAnalysis" );
	qxGetVoxelTimeCourseOfVTC = (GetVoxelTimeCourseOfVTCPrototype)GETSYMADDRESS( hndl, "qxGetVoxelTimeCourseOfVTC" );
	qxGetVoxelTimeCourseOfVTCs = (GetVoxelTimeCourseOfVTCsPrototype)GETSYMADDRESS( hndl, "qxGetVoxelTimeCourseOfVTCs" );
	qxGetVoxelCoordsTimeCourseOfVTC = (GetVoxelCoordsTimeCourseOfVTCPrototype)GETSYMADDRESS( hndl, "qxGetVoxelCoordsTimeCourseOfVTC" );
	qxGetVoxelCoordsTimeCourseOfVTCs = (GetVoxelCoordsTimeCourseOfVTCsPrototype)GETSYMADDRESS( hndl, "qxGetVoxelCoordsTimeCourseOfVTCs" );

	// GLM
	qxGetGLMOfCurrentVMR = (GetGLMOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetGLMOfCurrentVMR" );

	// MASK
	qxLoadMaskToCurrentVMR = (LoadMaskToCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxLoadMaskToCurrentVMR" );
	qxGetMaskOfCurrentVMR = (GetMaskOfCurrentVMRPrototype)GETSYMADDRESS( hndl, "qxGetMaskOfCurrentVMR" );

	// SRF
	qxGetScene = (GetScenePrototype)GETSYMADDRESS( hndl, "qxGetScene" );
	qxGetCurrentSRF = (GetCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetCurrentSRF" );
	qxReleaseSRFAccess = (ReleaseSRFAccessPrototype)GETSYMADDRESS( hndl, "qxReleaseSRFAccess" );
	qxSaveCurrentSRF = (SaveCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveCurrentSRF" );
	qxSetCurrentSRF = (SetCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSetCurrentSRF" );

	// MTC
	qxLinkMTCToCurrentSRF = (LinkMTCToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxLinkMTCToCurrentSRF" );
	qxGetMTCOfCurrentSRF = (GetMTCOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetMTCOfCurrentSRF" );
	qxSetMTCParametersOfCurrentSRF = (SetMTCParametersOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSetMTCParametersOfCurrentSRF" );
	qxCreateMTCForCurrentSRF = (CreateMTCForCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxCreateMTCForCurrentSRF" );
	qxSaveMTCOfCurrentSRF = (SaveMTCOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveMTCOfCurrentSRF" );

	// Multi-MTC
	qxClearMultiMTCRefsOfCurrentSRF = (ClearMultiMTCRefsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxClearMultiMTCRefsOfCurrentSRF" );
	qxAddMultiMTCRefToCurrentSRF = (AddMultiMTCRefToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxAddMultiMTCRefToCurrentSRF" );
	qxGetVertexTimeCourseOfMTC = (GetVertexTimeCourseOfMTCPrototype)GETSYMADDRESS( hndl, "qxGetVertexTimeCourseOfMTC" );
	qxGetVertexTimeCourseOfAllMTCs = (GetVertexTimeCourseOfAllMTCsPrototype)GETSYMADDRESS( hndl, "qxGetVertexTimeCourseOfAllMTCs" );

	// SMP
	qxGetSMPsOfCurrentSRF = (GetSMPsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetSMPsOfCurrentSRF" );
	qxGetSMPOfCurrentSRF = (GetSMPOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetSMPOfCurrentSRF" );
	qxSetSMPParametersOfCurrentSRF = (SetSMPParametersOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSetSMPParametersOfCurrentSRF" );
	qxLoadSMPsToCurrentSRF = (LoadSMPsToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxLoadSMPsToCurrentSRF" );
	qxSaveSMPsOfCurrentSRF = (SaveSMPsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveSMPsOfCurrentSRF" );
	qxCreateSMPForCurrentSRF = (CreateSMPForCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxCreateSMPForCurrentSRF" );
	qxDeleteSMPsOfCurrentSRF = (DeleteSMPsOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxDeleteSMPsOfCurrentSRF" );
	qxOverlaySMPsOnSRF = (OverlaySMPsOnSRFPrototype)GETSYMADDRESS( hndl, "qxOverlaySMPsOnSRF" );

	// SSM
	qxGetSSMOfCurrentSRF = (GetSSMOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxGetSSMOfCurrentSRF" );
	qxLoadSSMToCurrentSRF = (LoadSSMToCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxLoadSSMToCurrentSRF" );
	qxSaveSSMOfCurrentSRF = (SaveSSMOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxSaveSSMOfCurrentSRF" );
	qxCreateSSMForCurrentSRF = (CreateSSMForCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxCreateSSMForCurrentSRF" );
	qxDeleteSSMOfCurrentSRF = (DeleteSSMOfCurrentSRFPrototype)GETSYMADDRESS( hndl, "qxDeleteSSMOfCurrentSRF" );

	// PRT
	qxClearProtocol = (ClearProtocolPrototype)GETSYMADDRESS( hndl, "qxClearProtocol" );
	qxLoadProtocol = (LoadProtocolPrototype)GETSYMADDRESS( hndl, "qxLoadProtocol" );
	qxSaveProtocol = (SaveProtocolPrototype)GETSYMADDRESS( hndl, "qxSaveProtocol" );
	qxGetProtocol = (GetProtocolPrototype)GETSYMADDRESS( hndl, "qxGetProtocol" );
	qxSetProtocol = (SetProtocolPrototype)GETSYMADDRESS( hndl, "qxSetProtocol" );
	qxGetProtocolCondition = (GetProtocolConditionPrototype)GETSYMADDRESS( hndl, "qxGetProtocolCondition" );
	qxAddProtocolCondition = (AddProtocolConditionPrototype)GETSYMADDRESS( hndl, "qxAddProtocolCondition" );
	qxAddProtocolConditionInterval = (AddProtocolConditionIntervalPrototype)GETSYMADDRESS( hndl, "qxAddProtocolConditionInterval" );

	// Predictor and design matrix modelling functions
	qxClearDesignMatrix = (ClearDesignMatrixPrototype)GETSYMADDRESS( hndl, "qxClearDesignMatrix" );
	qxAddPredictor = (AddPredictorPrototype)GETSYMADDRESS( hndl, "qxAddPredictor" );
	qxSetPredictorValuesFromCondition = (SetPredictorValuesFromConditionPrototype)GETSYMADDRESS( hndl, "qxSetPredictorValuesFromCondition" );
	qxSetPredictorValuesFromConditionName = (SetPredictorValuesFromConditionNamePrototype)GETSYMADDRESS( hndl, "qxSetPredictorValuesFromConditionName" );
	qxApplyTwoGammaHRFToPredictor = (ApplyTwoGammaHRFToPredictorPrototype)GETSYMADDRESS( hndl, "qxApplyTwoGammaHRFToPredictor" );
	qxApplyTwoGammaHRFToPredictorWithName = (ApplyTwoGammaHRFToPredictorWithNamePrototype)GETSYMADDRESS( hndl, "qxApplyTwoGammaHRFToPredictorWithName" );
	qxApplyBoyntonHRFToPredictor = (ApplyBoyntonHRFToPredictorPrototype)GETSYMADDRESS( hndl, "qxApplyBoyntonHRFToPredictor" );
	qxApplyBoyntonHRFToPredictorWithName = (ApplyBoyntonHRFToPredictorWithNamePrototype)GETSYMADDRESS( hndl, "qxApplyBoyntonHRFToPredictorWithName" );
	qxGetPredictor = (GetPredictorPrototype)GETSYMADDRESS( hndl, "qxGetPredictor" );
	qxGetPredictorByName = (GetPredictorByNamePrototype)GETSYMADDRESS( hndl, "qxGetPredictorByName" );
	qxLoadSingleStudyGLMDesignMatrix = (LoadSingleStudyGLMDesignMatrixPrototype)GETSYMADDRESS( hndl, "qxLoadSingleStudyGLMDesignMatrix" );
	qxSaveSingleStudyGLMDesignMatrix = (SaveSingleStudyGLMDesignMatrixPrototype)GETSYMADDRESS( hndl, "qxSaveSingleStudyGLMDesignMatrix" );

	// VOI
	qxGetVOIs = (GetVOIsPrototype)GETSYMADDRESS( hndl, "qxGetVOIs" );
	qxGetVOI  = (GetVOIPrototype)GETSYMADDRESS( hndl, "qxGetVOI" );

	// POI
	qxGetPOIs = (GetPOIsPrototype)GETSYMADDRESS( hndl, "qxGetPOIs" );
	qxGetPOI  = (GetPOIPrototype)GETSYMADDRESS( hndl, "qxGetPOI" );

	// get dlg parameters for special plugins with dialog provided by BrainVoyagerQX
	qxGetDlgStringParameter = (GetDlgStringParameterPrototype)GETSYMADDRESS( hndl, "qxGetDlgStringParameter" );
	qxGetDlgIntegerParameter = (GetDlgIntegerParameterPrototype)GETSYMADDRESS( hndl, "qxGetDlgIntegerParameter" );
	qxGetDlgFloatParameter = (GetDlgFloatParameterPrototype)GETSYMADDRESS( hndl, "qxGetDlgFloatParameter" );
	qxSetDlgStringOutputParameter = (SetDlgStringOutputParameterPrototype)GETSYMADDRESS( hndl, "qxSetDlgStringOutputParameter" );
	qxSetDlgIntegerOutputParameter = (SetDlgIntegerOutputParameterPrototype)GETSYMADDRESS( hndl, "qxSetDlgIntegerOutputParameter" );
	qxSetDlgFloatOutputParameter = (SetDlgFloatOutputParameterPrototype)GETSYMADDRESS( hndl, "qxSetDlgFloatOutputParameter" );
	qxLaunchICADialog = (LaunchICADialogPrototype)GETSYMADDRESS( hndl, "qxLaunchICADialog" );

	return true;
}

bool checkVersionGreaterEqualVersion(int MajorA, int MinorA, int SubMinorA, int MajorB, int MinorB, int SubMinorB)
{
	if(MajorA > MajorB)
		return true;

	if(MajorB > MajorA)
		return false;

	// Major is equal, Minor decides

	if(MinorA > MinorB)
		return true;

	if(MinorB > MinorA)
		return false;

	// Minor is equal, SubMinor decides

	if(SubMinorA >= SubMinorB)
		return true;

	// B smaller
	return false;
}
